; Started 32-bit kernel on August 21 2004
[BITS 32]
[ORG 1000h]	; Loaded at 1000h
%define	LSHIFT	02ah
%define	RSHIFT	036h

start:
mov bl,07h
call ClearScr

mov esi,LoadMsg
call PrintPMode

mov al,00010001b ;ICW4 is sent, cascaded PICs, edge triggering
out 20h,al       ;sending ICW1 to master
out 0A0h,al      ;sending ICW1 to slave

mov al,20h			; int 20h is base for PIC1
out 21h,al
mov al,28h			; int 28h is base for PIC2
out 0A1h,al

mov al,00000100b		; IR line 2 is connected to PIC2 (Slave)
out 21h,al
mov al,00000010b		; IRQ line 2 of PIC1 is connected to PIC2
out 0A1h,al

mov al,01h			; PIC1 is Master
out 21h,al	
out 0A1h,al			; PIC 2 is slave

mov	ecx,1000h
wait_8259:

call	Delay32
dec	ecx
jnz	wait_8259

mov al,11111100b	; mask all interrupts except the first two (IRQ 0,1 and 2)
out 21h,al
mov al,0FFh		; mask all ints on PIC 2 ( 0FFh = 11111111b )
out 0A1h,al

mov al,20h
out 64h,al	; Tell keyboard we want to read cmd byte
call Empty8042
in al,60h	; Read in command byte
or al,01h	; set bit 0 (keyboard interrupts enabled)
;or al,03h	; set bit 0 & 1(keyboard & mouse interrupts enabled)
;out 64h,al

lidt [idt_desc]
sti
xor al,al
mov esi,Loaded
int 30h

xor al,al		; call the RooOS
mov esi,Prompt	; print function
int 30h		;
@cmdloop:
;	mov al,1		; call the RooOs getch
;	int 30h		; function
	call getch

	cmp al,0Dh
	je Cmd
;	or al,al
;	je @cmdloop
	
;	xor al,al		; call the RooOS
;	mov esi,Prompt	; print function
;	int 30h		;	

	mov edx,[CursorPos]
	mov [CmdBuffer+edx],al

	inc dword [CursorPos]
	jmp @cmdloop
Cmd:
	mov dword[CursorPos],0
	mov esi,CmdList
	xor eax,eax
	xor ecx,ecx
	xor ebx,ebx
FindCmd:
	mov edi,CmdBuffer
	mov al,byte[edi]
	test al,al
	je ClearBuff
	lodsb
	test al,al
	je BadCmd
	mov cl,al
	push esi
	repe cmpsb
	je FindAddr
	pop esi

	add esi,eax
	inc ebx
	jmp FindCmd
FindAddr:
	shl ebx,2
	mov esi,CmdAddr
	add esi,ebx
	mov esi,[esi]
	jmp esi
;	add ebx,CmdAddr
;	mov esi,[ebx]

BadCmd:
	xor eax,eax
	mov esi,BadCommand
	int 30h

ClearBuff:
	xor al,al		; call the RooOS
	mov esi,Prompt	; print function
	int 30h		;

	mov edi,CmdBuffer
	mov ecx,64
	xor eax,eax
	rep stosd
	jmp @cmdloop

SayHello:
	mov esi,HelloMsg
	xor eax,eax
	int 30h
	jmp ClearBuff

Timer:
	cli
	pushad
	push ds
	push es

	inc dword[Ticks]
	mov byte[ds:0B8000h],'R'
	mov eax,[Ticks]
	mov byte[ds:0B8001h],al

	mov al,20h
	out 20h,al	; Send EOI (End of Interrupt)

	pop es
	pop ds
	popad
	iret

Syscall_Handler:
	push esi
	push edi

	or al,al
	je @30_0

	cmp al,1
	je @30_1

	call ClearScr
	jmp @30_exit
@30_0:
	call PrintPMode
	jmp @30_exit
@30_1:
	call getch
@30_exit:
	push eax	; preserve eax
	mov al,20h	;
	out 20h,al	; Send EOI (End of Interrupt)
	pop eax
	
	pop edi
	pop esi
	iret

PrintPMode:
	cld
	push ecx
	mov edi,0B8000h
	mov edx,[TextPtr]
	shl edx,1
	add edi,edx
	mov ah,[ds:esi]	; Get text attributes
	inc esi
	strloop:
		mov al,[ds:esi]
		test al,al
		je endstr
		inc esi		; faster than lodsb
		cmp al,0Dh
		je _CR
	put:	mov word [es:edi],ax
		inc dword [TextPtr]
		add edi,2
		jmp strloop
	_CR:	call PutCR
		mov eax,[TextPtr]
		shl eax,1
		mov edi,0B8000h
		add edi,eax

		jmp strloop
	endstr:
	call MoveCursor
	pop ecx
	ret

Empty8042:
	in al,64h
	test al,1
	jz empty1
	in al,060h
	jmp Empty8042
  empty1:
	test al,2
	jnz Empty8042
	retn 4

Delay32:		; borrowed from SolarOs
	jmp delay_01
delay_01:
	jmp delay_02
delay_02:
	ret

Wait_Read_8042:
	call Delay32

	in al,064h
	and al,01b
	jnz have_data
	jmp Wait_Read_8042
have_data:
	ret

Wait_Write_8042:
	call Delay32

	in al,64h
	and al,10b
	jnz Wait_Write_8042
	ret

PutCR:
	push eax	; save all the regs
	push edx
	push ecx
	mov eax,[TextPtr]	; get current VGA CursorPos
	xor edx,edx		; zero edx
	mov ecx,80		; divide the cursur pos by 80
	push ecx
	div ecx		;
	pop ecx
	sub ecx,edx
	add [TextPtr],ecx	; add the remainder to current pos
	mov eax,[TextPtr]
	call MoveCursor
	pop ecx
	pop edx
	pop eax
	ret

PutBkSpce:
	cmp dword[CursorPos],0
	je @bk_x
	dec dword[CursorPos]
	dec dword[TextPtr]
	dec dword[keyPos]
	sub edi,2

	mov word[ds:edi],20h
@bk_x:call MoveCursor
	ret

MoveCursor:
	push ax
	push cx
	push dx

	mov cx,[TextPtr]
	mov bx,cx		; save it in bx

	mov dx,03D4h
	mov ax,14
	out dx,ax	; out 03D4h,14

	inc dx
	shr bx,8
	mov ax,bx
	out dx,ax	; out 03D5h,(TextPtr >> 8)

	dec dx
	mov ax,15
	out dx,ax	; out 03D4h,15

	inc dx
	mov ax,cx
	out dx,ax	; out 03D5h,TextPtr

	pop dx
	pop cx
	pop ax
	ret

GoToXY:	; eax = row, ebx = col, pos = 2*(row*80+col)
	mov ecx,eax	; take another copy in ecx
	shl eax,6	; multiply eax by 64
	shl ecx,4	; multiply ecx by 16
	add eax,ecx	; add both together to get row * 80 as 64 +16 = 80
	add eax,ebx	; add col no.
	inc eax
	mov [TextPtr],eax
	call MoveCursor
	ret

ClearScr:
	cld
	mov edi,0B8000h
	xor ecx,ecx
	mov al," "
	mov ah,bl
	clrloop:
		cmp ecx,2000
		je endcloop
		inc ecx
		mov word [ds:edi],ax
		add edi,2
		jmp clrloop
	endcloop:
	mov dword [TextPtr],0
	ret

Keyboard:
	cli
	pushad
	pushf
	xor eax,eax
loop1:
	in al,64h
	and al,01h
	jz loop1
	in al,60h

	mov ebx,eax

	cmp al,0E0h		; if special key
	je @kb_e		; set previous scancode
	cmp al,0E1h		; to it
	je @kb_e		;

	or al,al		;
	je @kb_f		;
	cmp al,0FFh		;
	je @kb_f		;
	
	cmp al,02Ah		; check for shift
	je @kb_ls		;
	cmp al,036h		;
	je @kb_rs		;

	cmp al,0AAh		; check for shift
	je @kb_lS		; release
	cmp al,0B6h		;
	je @kb_rS		;

	jmp @kb_1

@kb_f:xor al,al
@kb_e:
	mov byte[prev_scode],al
	jmp @kb_0
@kb_s:mov byte[shift],1
	jmp @kb_0
@kb_S:mov byte[shift],0
	jmp @kb_0

@kb_ls:mov byte[lshift],1
	jmp @kb_0
@kb_rs:mov byte[rshift],1
	jmp @kb_0

@kb_lS:mov byte[lshift],0
	jmp @kb_0
@kb_rS:mov byte[rshift],0
	jmp @kb_0

@kb_1:cmp al,02h		; check if keys b/w
	jge $+7		; 02h and 0Dh
	jmp @kb_0		; which are the row of keys above
	cmp al,0Eh		; the alphabets upto backspace
	jle @kb		;

	cmp al,10h		; check if keys b/w
	jge $+7		; 10h and 1Bh
	jmp @kb_0		; which are q w e r t y u i o p [ ]
	cmp al,1Ch		; 
	jle @kb		;

	cmp al,1Eh		; check if keys b/w
	jge $+7		; 1Eh and 28h
	jmp @kb_0		; which are :
	cmp al,28h		; a s d f g h j k l ; '
	jle @kb		;

	cmp al,2Bh		; check if keys b/w
	jge $+7		; 2Ch and 34h
	jmp @kb_0		; which are :
	cmp al,35h		; \ z x c v b n m , . /
	jle @kb		;

	cmp al,39h
	je @kb

	cmp byte[prev_scode],0
	jne @kb_E

	jmp @kb_0
	
@kb:	call getKeyChar

	mov edx,keyBuffer
	add edx,[keyPos]
	mov byte[edx],al
	inc dword[keyBufSize]
	inc dword[keyPos]

@kb1:	;call putKeyChar
	jmp @kb_0

@kb_E:mov byte[prev_scode],0
	mov edx,Keymap_E0
	mov al,byte[edx+eax]
	call useSpecKeys
	
@kb_0:call MoveCursor
	mov al,20h
	out 20h,al	; Send EOI (End of Interrupt)

	popf
	popad
	iret

getKeyChar:
	pushf
	push edx

	cmp byte[lshift],1
	je @gk_shift
	cmp byte[rshift],1
	je @gk_shift
	mov edx,Keymap_NoCaps
	jmp @gk_exit
@gk_shift:
	mov edx,Keymap_Caps
@gk_exit:
	mov al,[edx+eax]
	mov byte[ds:0B8080h],al
	mov byte[ds:0B8081h],al
	pop edx
	popf
	ret

putKeyChar:
	mov edi,0B8000h
	mov edx,[TextPtr]
	shl edx,1
	add edi,edx
	cmp al,0Dh		; enter
	je @pkc_cr
	cmp al,08h		; backspace
	je @pkc_bk
	mov ah,0Eh
	mov word [edi],ax
	inc dword [TextPtr]
	jmp @pkc_exit
@pkc_cr:
	call PutCR
	jmp @pkc_exit
@pkc_bk:
	call PutBkSpce
@pkc_exit:
	ret

useSpecKeys:
	cmp al,0Dh
	je short $+7
	jmp @use_exit
	call PutCR
@use_exit:
	ret

getch:
	xor eax,eax
	xor ecx,ecx
	xor edx,edx
@g1:	cmp dword[keyBufSize],0
	je @g1
	mov edx,keyBuffer
	add edx,[keyPos]
	dec edx
	mov al,byte[edx]
	dec dword[keyBufSize]
	dec dword[keyPos]

	mov edx,keyBuffer
	inc edx
	mov esi,edx
	mov edi,keyBuffer
	mov ecx,99
	rep movsb

@g3:	call putKeyChar
	ret

%include	"idt.inc"
%include	"keys.inc"

LoadMsg	db 0Eh,"RooOS Kernel is loading......",0Dh,0
Loaded	db 0Eh,"Loaded Ok",0Dh,0
BadCommand	db 0Eh,"Bad Command entered",0Dh,0
HelloMsg	db 0Eh,"Hello!!",0Dh,0
Prompt	db 0Eh,"Roo:-> ",0

Keymap_NoCaps db 0,ESCAPE,"1234567890-=",BACKSPACE
		db "	qwertyuiop[]",ENTER,0
		db "asdfghjkl;'`",0,'\'
		db "zxcvbnm,./",0,0,0,' ',0
		db KEY_F1,KEY_F2,KEY_F3,KEY_F4,KEY_F5,KEY_F6,KEY_F7,KEY_F8,KEY_F9,KEY_F10
		db 0,0,KEY_HOME,KEY_UP,KEY_PGUP,'-',KEY_LEFT,'5',KEY_RIGHT,'+',KEY_END,KEY_DOWN
		db KEY_PGDN,KEY_INS,KEY_DEL,0,0,0,KEY_F11,KEY_F12,0

Keymap_Caps db 0,ESCAPE, '!@#$%^&*()_+' ,BACKSPACE
		db '	QWERTYUIOP{}',ENTER,0
		db 'ASDFGHJKL:"~',0,'|'
		db 'ZXCVBNM<>?',0,0,0,' ',0
		db KEY_F1,KEY_F2,KEY_F3,KEY_F4,KEY_F5,KEY_F6,KEY_F7,KEY_F8,KEY_F9,KEY_F10
		db 0,0,KEY_HOME,KEY_UP,KEY_PGUP,'-',KEY_LEFT,'5',KEY_RIGHT,'+',KEY_END,KEY_DOWN
		db KEY_PGDN,KEY_INS,KEY_DEL,0,0,0,KEY_F11,KEY_F12

Keymap_E0	db 0, 0, 0, 0, 0, 0, 0, 0,
		db 0, 0, 0, 0, ENTER, KEY_RCTRL, 0, 0,
		db 0, 0, 0, 0, 0, 0, 0, 0,
		db 0, 0, 0, 0, 0, 0, 0, 0,
		db 0, 0, 0, 0, 0, 0, 0, 0,
		db 0, 0, 0, 0, 0, 0, KEY_BREAK, KEY_HOME,
		db KEY_UP, KEY_PGUP, 0, KEY_LEFT, 0, KEY_RIGHT, 0, KEY_END,
		db KEY_DOWN, KEY_PGDN, KEY_INS, KEY_DEL, 0, 0, 0, 0,
		db 0, 0, 0, 0, 0, 0, 0, 0,
		db 0, 0, 0, 0, 0, 0, 0, 0,
		db 0, 0, 0, 0, 0, 0, 0, 0,
		db 0, 0, 0, 0, 0, 0, 0, 0,
		db 0, 0, 0, 0, 0, 0, 0, 0,
		db 0, 0, 0, 0, 0, 0, 0, 0,
		db 0, 0, 0, 0, 0, 0, 0, 0,
		db 0, 0, 0, 0, 0, 0, 0, 0


CmdList	db 5,"hello",0
CmdAddr	dd SayHello,0

TextPtr	dd 0
Ticks		dd 0


CmdBuffer	times 256 db 0
CursorPos	dd 0

keyBuffer	times 100 db 0
keyPos	dd 0
keyBufSize	dd 0

prev_scode	db 0
shift		db 0
lshift	db 0
rshift	db 0

